@@ -63,7 +63,7 @@ parse :: SepFormat -> InputOpts -> FilePath -> Handle -> ExceptT String IO Journ
 parse sep iopts f h = do
   rules <- readRules $ getRulesFile f (mrules_file_ iopts)
   mencoding <- rulesEncoding rules
-  csvtext <- lift $ readHandlePortably' mencoding h
+  csvtext <- lift $ hGetContentsPortably mencoding h
   readJournalFromCsv rules f csvtext (Just sep)
   -- apply any command line account aliases. Can fail with a bad replacement pattern.
   >>= liftEither . journalApplyAliases (aliasesFromOpts iopts)
